/* Copyright 2019 DmNosachev
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include QMK_KEYBOARD_H


const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {

/* 
 * ,---------------------------------------------------------------------------------------.
 * |   ,----------------.  ,-----------------------.  ,--------------.          | up |     |
 * |   |vol- |vol+| mute|  | prev| play| stop| next|  |prsc|scrlk|ins|     |left|down|right|
 * |---------------------------------------------------------------------------------------|
 * | ~`|  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |  9  |  0  |  -  |  = |   BSp| L1  |
 * |---------------------------------------------------------------------------------------|
 * |Tab/L1 | Q  |  W  |  E  |  R  |  T  |  Y  |  U  |  I  |  O  |  P  |  [  |  ]  |  |  \| |
 * |---------------------------------------------------------------------------------------|
 * |Ctrl  |  A  |  S  |  D  |  F  |  G  |  H  |  J  |  K  |  L  |  ;  |  "  | enter  | gui |
 * |---------------------------------------------------------------------------------------'
 * |  Shift  |  Z  |  X  |  C  |  V  |  B  |  N  |  M  |  <  |  >  |  ?  |   Shift   |
 * `---------------------------------------------------------------------------------'
 *        | Alt  |                     Space                         | Caps  |
 *        `------------------------------------------------------------------'
 */
 
  [0] = LAYOUT(
		        KC_VOLD, KC_VOLU, KC_MUTE,      KC_MPRV, KC_MPLY, KC_MSTP, KC_MNXT,    KC_PSCR, KC_SLCK, KC_INS,      KC_LEFT, KC_DOWN, KC_UP, KC_RGHT,
		KC_GRV,     KC_1, KC_2, KC_3, KC_4, KC_5, KC_6, KC_7, KC_8,    KC_9,   KC_0,    KC_MINS, KC_EQL,  KC_BSPC, MO(1),
		LT(1, KC_TAB),  KC_Q, KC_W, KC_E, KC_R, KC_T, KC_Y, KC_U, KC_I,    KC_O,   KC_P,    KC_LBRC, KC_RBRC,   KC_BSLS,
		KC_LCTL, KC_A, KC_S, KC_D, KC_F, KC_G, KC_H, KC_J, KC_K,    KC_L,   KC_SCLN, KC_QUOT, KC_ENT,              KC_RGUI,
		KC_LSFT, KC_Z, KC_X, KC_C, KC_V, KC_B, KC_N, KC_M, KC_COMM, KC_DOT, KC_SLSH, KC_RSFT,
		  KC_LALT,                              KC_SPC,                                          KC_CAPS
  ),

/* 
 * sect = section sign, Unicode 0x00A7
 * ,---------------------------------------------------------------------------------------.
 * |   ,----------------.  ,-----------------------.  ,--------------.          |pgup|     |
 * |   |     |    |     |  |     |     |      |    |  |  |reset|pause|     |home|pgdn| end |
 * |---------------------------------------------------------------------------------------|
 * |Esc| F1  | F2  | F3  | F4  | F5  | F6  | F7  | F8  | F9 | F10 | F11 | F12 |  del |     |
 * |---------------------------------------------------------------------------------------|
 * |      | mlb | mup | mrb | ms+ |     |     |     |  up |     |     |     |     |  |ctrl |
 * |---------------------------------------------------------------------------------------|
 * |      |  ml | mdn | mr  | ms- |     |     | left| down|right|     |     |        |     |
 * |---------------------------------------------------------------------------------------'
 * |         |     |     |     |     |     |     |     |     |     |     |           |   
 * `---------------------------------------------------------------------------------'
 *       |      |                                                   |       |
 *       `------------------------------------------------------------------'
 */
 
  [1] = LAYOUT(
		      	_______, _______, _______,      _______, _______, _______, _______,    _______, RESET,  KC_PAUS,    KC_HOME, KC_PGDN, KC_PGUP, KC_END,
		_______, KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5,   KC_F6,   KC_F7,   KC_F8,   KC_F9,   KC_F10,  KC_F11,  KC_F12,  KC_DEL,  _______,
		_______, KC_BTN1, KC_MS_U, KC_BTN2, KC_WH_U ,_______, _______, _______, KC_UP,   _______, _______, _______, _______,          KC_RCTRL,
		_______, KC_MS_L, KC_MS_D, KC_MS_R, KC_WH_D, _______, _______, KC_LEFT, KC_DOWN, KC_RGHT, _______, _______, _______,          _______,
		_______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
		  _______,                                _______,                                          _______
  ),
  [2] = LAYOUT(
		      	 _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
		_______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
		_______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______,
		_______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______,
		_______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
		  _______,                                _______,                                          _______
  ),
  [3] = LAYOUT(
		      	 _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
		_______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
		_______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______,
		_______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______,
		_______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
		  _______,                                _______,                                          _______
  )
};
