/* Copyright 2019
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Sendstring lookup tables for Hungarian layouts

#pragma once

#include "keymap_hungarian.h"
#include "quantum.h"

// clang-format off

const uint8_t ascii_to_shift_lut[16] PROGMEM = {
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),

    KCLUT_ENTRY(0, 1, 1, 0, 0, 1, 0, 1),
    KCLUT_ENTRY(1, 1, 0, 1, 0, 0, 0, 1),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 1, 0, 0, 1, 0, 1),
    KCLUT_ENTRY(0, 1, 1, 1, 1, 1, 1, 1),
    KCLUT_ENTRY(1, 1, 1, 1, 1, 1, 1, 1),
    KCLUT_ENTRY(1, 1, 1, 1, 1, 1, 1, 1),
    KCLUT_ENTRY(1, 1, 1, 0, 0, 0, 0, 1),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0)
};

const uint8_t ascii_to_altgr_lut[16] PROGMEM = {
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),

    KCLUT_ENTRY(0, 0, 0, 1, 1, 0, 1, 0),
    KCLUT_ENTRY(0, 0, 1, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 1, 1, 0, 1, 0),
    KCLUT_ENTRY(1, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 1, 1, 1, 1, 0),
    KCLUT_ENTRY(1, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 1, 1, 1, 1, 0)
};

const uint8_t ascii_to_keycode_lut[128] PROGMEM = {
    // NUL   SOH      STX      ETX      EOT      ENQ      ACK      BEL
    XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,
    // BS    TAB      LF       VT       FF       CR       SO       SI
    KC_BSPC, KC_TAB,  KC_ENT,  XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,
    // DLE   DC1      DC2      DC3      DC4      NAK      SYN      ETB
    XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,
    // CAN   EM       SUB      ESC      FS       GS       RS       US
    XXXXXXX, XXXXXXX, XXXXXXX, KC_ESC,  XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,

    //       !        "        #        $        %        &        '
    KC_SPC,  HU_4,    HU_2,    HU_X,    HU_EACU, HU_5,    HU_C,    HU_1,
    // (     )        *        +        ,        -        .        /
    HU_8,    HU_9,    HU_MINS, HU_3,    HU_COMM, HU_MINS, HU_DOT,  HU_6,
    // 0     1        2        3        4        5        6        7
    HU_0,    HU_1,    HU_2,    HU_3,    HU_4,    HU_5,    HU_6,    HU_7,
    // 8     9        :        ;        <        =        >        ?
    HU_8,    HU_9,    HU_DOT,  HU_COMM, HU_M,    HU_7,    HU_DOT,  HU_COMM,
    // @     A        B        C        D        E        F        G
    HU_V,    HU_A,    HU_B,    HU_C,    HU_D,    HU_E,    HU_F,    HU_G,
    // H     I        J        K        L        M        N        O
    HU_H,    HU_I,    HU_J,    HU_K,    HU_L,    HU_M,    HU_N,    HU_O,
    // P     Q        R        S        T        U        V        W
    HU_P,    HU_Q,    HU_R,    HU_S,    HU_T,    HU_U,    HU_V,    HU_W,
    // X     Y        Z        [        \        ]        ^        _
    HU_X,    HU_Y,    HU_Z,    HU_F,    HU_Q,    HU_G,    HU_3,    HU_MINS,
    // `     a        b        c        d        e        f        g
    HU_7,    HU_A,    HU_B,    HU_C,    HU_D,    HU_E,    HU_F,    HU_G,
    // h     i        j        k        l        m        n        o
    HU_H,    HU_I,    HU_J,    HU_K,    HU_L,    HU_M,    HU_N,    HU_O,
    // p     q        r        s        t        u        v        w
    HU_P,    HU_Q,    HU_R,    HU_S,    HU_T,    HU_U,    HU_V,    HU_W,
    // x     y        z        {        |        }        ~        DEL
    HU_X,    HU_Y,    HU_Z,    HU_B,    HU_W,    HU_N,    HU_1,    KC_DEL
};
