/* Copyright 2019
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Sendstring lookup tables for Norwegian layouts

#pragma once

#include "keymap_norwegian.h"
#include "quantum.h"

// clang-format off

const uint8_t ascii_to_shift_lut[16] PROGMEM = {
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),

    KCLUT_ENTRY(0, 1, 1, 1, 0, 1, 1, 0),
    KCLUT_ENTRY(1, 1, 1, 0, 0, 0, 0, 1),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 1, 1, 0, 1, 1, 1),
    KCLUT_ENTRY(0, 1, 1, 1, 1, 1, 1, 1),
    KCLUT_ENTRY(1, 1, 1, 1, 1, 1, 1, 1),
    KCLUT_ENTRY(1, 1, 1, 1, 1, 1, 1, 1),
    KCLUT_ENTRY(1, 1, 1, 0, 0, 0, 1, 1),
    KCLUT_ENTRY(1, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0)
};

const uint8_t ascii_to_altgr_lut[16] PROGMEM = {
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),

    KCLUT_ENTRY(0, 0, 0, 0, 1, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(1, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 1, 0, 1, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 0, 0, 0, 0, 0),
    KCLUT_ENTRY(0, 0, 0, 1, 0, 1, 1, 0)
};

const uint8_t ascii_to_keycode_lut[128] PROGMEM = {
    // NUL   SOH      STX      ETX      EOT      ENQ      ACK      BEL
    XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,
    // BS    TAB      LF       VT       FF       CR       SO       SI
    KC_BSPC, KC_TAB,  KC_ENT,  XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,
    // DLE   DC1      DC2      DC3      DC4      NAK      SYN      ETB
    XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,
    // CAN   EM       SUB      ESC      FS       GS       RS       US
    XXXXXXX, XXXXXXX, XXXXXXX, KC_ESC,  XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,

    //       !        "        #        $        %        &        '
    KC_SPC,  NO_1,    NO_2,    NO_3,    NO_4,    NO_5,    NO_6,    NO_QUOT,
    // (     )        *        +        ,        -        .        /
    NO_8,    NO_9,    NO_QUOT, NO_PLUS, NO_COMM, NO_MINS, NO_DOT,  NO_7,
    // 0     1        2        3        4        5        6        7
    NO_0,    NO_1,    NO_2,    NO_3,    NO_4,    NO_5,    NO_6,    NO_7,
    // 8     9        :        ;        <        =        >        ?
    NO_8,    NO_9,    NO_DOT,  NO_COMM, NO_LABK, NO_0,    NO_LABK, NO_PLUS,
    // @     A        B        C        D        E        F        G
    NO_2,    NO_A,    NO_B,    NO_C,    NO_D,    NO_E,    NO_F,    NO_G,
    // H     I        J        K        L        M        N        O
    NO_H,    NO_I,    NO_J,    NO_K,    NO_L,    NO_M,    NO_N,    NO_O,
    // P     Q        R        S        T        U        V        W
    NO_P,    NO_Q,    NO_R,    NO_S,    NO_T,    NO_U,    NO_V,    NO_W,
    // X     Y        Z        [        \        ]        ^        _
    NO_X,    NO_Y,    NO_Z,    NO_8,    NO_BSLS, NO_9,    NO_DIAE, NO_MINS,
    // `     a        b        c        d        e        f        g
    NO_BSLS, NO_A,    NO_B,    NO_C,    NO_D,    NO_E,    NO_F,    NO_G,
    // h     i        j        k        l        m        n        o
    NO_H,    NO_I,    NO_J,    NO_K,    NO_L,    NO_M,    NO_N,    NO_O,
    // p     q        r        s        t        u        v        w
    NO_P,    NO_Q,    NO_R,    NO_S,    NO_T,    NO_U,    NO_V,    NO_W,
    // x     y        z        {        |        }        ~        DEL
    NO_X,    NO_Y,    NO_Z,    NO_7,    NO_PIPE, NO_0,    NO_DIAE, KC_DEL
};
